@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2009 Yoran Heling
+  Copyright (c) 2007-2010 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -55,6 +55,9 @@ dev_t curdev;            /* current device we're calculating on */
 int anpos;               /* position of the animation string */
 int curpathl = 0, lasterrl = 0;
 
+struct dir **links = NULL;
+int linksl, linkst;
+
 
 
 /* adds name to curpath */
@@ -84,6 +87,79 @@ void calc_leavepath() {
 }
 
 
+/* looks in the links list and adds the file when not found */
+int calc_hlink_add(struct dir *d) {
+  int i;
+  /* check the list */
+  for(i=0; i<linkst; i++)
+    if(links[i]->dev == d->dev && links[i]->ino == d->ino)
+      break;
+  /* found, do nothing and return the index */
+  if(i != linkst)
+    return i;
+  /* otherwise, add to list and return -1 */
+  if(++linkst > linksl) {
+    linksl *= 2;
+    if(!linksl) {
+      linksl = 64;
+      links = malloc(linksl*sizeof(struct dir *));
+    } else
+      links = realloc(links, linksl*sizeof(struct dir *));
+  }
+  links[linkst-1] = d;
+  return -1;
+}
+
+
+/* recursively checks a dir structure for hard links and fills the lookup array */
+void calc_hlink_init(struct dir *d) {
+  struct dir *t;
+
+  for(t=d->sub; t!=NULL; t=t->next)
+    calc_hlink_init(t);
+
+  if(!(d->flags & FF_HLNKC))
+    return;
+  calc_hlink_add(d);
+}
+
+
+/* checks an individual file and updates the flags and cicrular linked list,
+ * also updates the sizes of the parent dirs */
+void calc_hlink_check(struct dir *d) {
+  struct dir *t, *pt, *par;
+  int i;
+
+  d->flags |= FF_HLNKC;
+  i = calc_hlink_add(d);
+
+  /* found in the list? update hlnk */
+  if(i >= 0) {
+    t = d->hlnk = links[i];
+    if(t->hlnk != NULL)
+      for(t=t->hlnk; t->hlnk!=d->hlnk; t=t->hlnk)
+        ;
+    t->hlnk = d;
+  }
+
+  /* now update the sizes of the parent directories,
+   * This works by only counting this file in the parent directories where this
+   * file hasn't been counted yet, which can be determined from the hlnk list.
+   * XXX: This may not be the most efficient algorithm to do this */
+  for(i=1,par=d->parent; i&&par; par=par->parent) {
+    if(d->hlnk)
+      for(t=d->hlnk; i&&t!=d; t=t->hlnk)
+        for(pt=t->parent; i&&pt; pt=pt->parent)
+          if(pt==par)
+            i=0;
+    if(i) {
+      par->size += d->size;
+      par->asize += d->asize;
+    }
+  }
+}
+
+
 int calc_item(struct dir *par, char *name) {
   struct dir *t, *d;
   struct stat fs;
@@ -92,11 +168,12 @@ int calc_item(struct dir *par, char *name) {
     return 0;
 
   /* allocate dir and fix references */
-  d = calloc(sizeof(struct dir), 1);
+  d = calloc(SDIRSIZE+strlen(name), 1);
   d->parent = par;
   d->next = par->sub;
   par->sub = d;
-  d->name = malloc(strlen(name)+1);
+  if(d->next)
+    d->next->prev = d;
   strcpy(d->name, name);
 
 #ifdef __CYGWIN__
@@ -126,27 +203,29 @@ int calc_item(struct dir *par, char *name) {
   else if(S_ISDIR(fs.st_mode))
     d->flags |= FF_DIR;
 
-  /* update parent dirs */
+  /* update the items count of the parent dirs */
   if(!(d->flags & FF_EXL))
     for(t=d->parent; t!=NULL; t=t->parent)
       t->items++;
 
-  /* Provide the necessary information for hard link checking */
-  d->ino = fs.st_ino;
-  d->dev = fs.st_dev;
-  if(!S_ISDIR(fs.st_mode) && fs.st_nlink > 1)
-    d->flags |= FF_HLNKC;
-
   /* count the size */
   if(!(d->flags & FF_EXL || d->flags & FF_OTHFS)) {
     d->size = fs.st_blocks * S_BLKSIZE;
     d->asize = fs.st_size;
-    for(t=d->parent; t!=NULL; t=t->parent) {
-      t->size += d->size;
-      t->asize += d->asize;
-    }
+    /* only update the sizes of the parents if it's not a hard link */
+    if(S_ISDIR(fs.st_mode) || fs.st_nlink <= 1)
+      for(t=d->parent; t!=NULL; t=t->parent) {
+        t->size += d->size;
+        t->asize += d->asize;
+      }
   }
 
+  /* Hard link checking (also takes care of updating the sizes of the parents) */
+  d->ino = fs.st_ino;
+  d->dev = fs.st_dev;
+  if(!S_ISDIR(fs.st_mode) && fs.st_nlink > 1)
+    calc_hlink_check(d);
+
   return 0;
 }
 
@@ -167,7 +246,7 @@ int calc_dir(struct dir *dest, char *name) {
 
   /* open & chdir into directory */
   if((dir = opendir(name)) == NULL || chdir(name) < 0) {
-    if(lasterrl < (int)strlen(curpath)) {
+    if(lasterrl <= (int)strlen(curpath)) {
       lasterrl = strlen(curpath)+1;
       lasterr = realloc(lasterr, lasterrl);
     }
@@ -309,6 +388,14 @@ int calc_process() {
   struct dir *t;
   int n;
 
+  /* create initial links array */
+  linksl = linkst = 0;
+  if(orig) {
+    for(t=orig; t->parent!=NULL; t=t->parent)
+      ;
+    calc_hlink_init(t);
+  }
+
   /* check root directory */
   if((path = path_real(curpath)) == NULL) {
     failed = 1;
@@ -346,15 +433,15 @@ int calc_process() {
   }
 
   /* initialize parent dir */
-  t = (struct dir *) calloc(1, sizeof(struct dir));
+  n = orig ? strlen(orig->name) : strlen(path)+strlen(name)+1;
+  t = (struct dir *) calloc(1, SDIRSIZE+n);
   t->size = fs.st_blocks * S_BLKSIZE;
   t->asize = fs.st_size;
   t->flags |= FF_DIR;
   if(orig) {
-    t->name = malloc(strlen(orig->name)+1);
     strcpy(t->name, orig->name);
+    t->parent = orig->parent;
   } else {
-    t->name = malloc(strlen(path)+strlen(name)+2);
     t->name[0] = 0;
     if(strcmp(path, "/"))
       strcpy(t->name, path);
@@ -366,9 +453,17 @@ int calc_process() {
   root = t;
   curdev = fs.st_dev;
 
+  /* make sure to count this directory entry in its parents at this point */
+  if(orig)
+    for(t=root->parent; t!=NULL; t=t->parent) {
+      t->size += root->size;
+      t->asize += root->asize;
+      t->items += 1;
+    }
+
   /* update curpath */
   if(strcmp(name, ".")) {
-    if((int)strlen(path)+1 > curpathl) {
+    if(curpathl <= (int)strlen(path)) {
       curpathl = strlen(path)+1;
       curpath = realloc(curpath, curpathl);
     }
@@ -384,6 +479,11 @@ int calc_process() {
     free(name);
   free(path);
 
+  if(links != NULL) {
+    free(links);
+    links = NULL;
+  }
+
   /* success */
   if(!n && !failed) {
     if(root->sub == NULL) {
@@ -395,28 +495,19 @@ int calc_process() {
 
     /* update references and free original item */
     if(orig) {
-      root->parent = orig->parent;
       root->next = orig->next;
-      for(t=root->parent; t!=NULL; t=t->parent) {
-        t->size += root->size;
-        t->asize += root->asize;
-        t->items += root->items+1;
-      }
-
-      if(orig->parent) {
-        t = orig->parent->sub;
-        if(t == orig)
-          orig->parent->sub = root;
-        else if(t != NULL)
-          for(; t->next!=NULL; t=t->next)
-            if(t->next == orig)
-              t->next = root;
-      }
+      root->prev = orig->prev;
+      if(root->parent && root->parent->sub == orig)
+        root->parent->sub = root;
+      if(root->prev)
+        root->prev->next = root;
+      if(root->next)
+        root->next->prev = root;
+      orig->next = orig->prev = NULL;
       freedir(orig);
     }
-
-    link_del(root);
     browse_init(root->sub);
+    dirlist_top(-3);
     return 0;
   }
 
@@ -436,12 +527,11 @@ calc_fail:
 
 void calc_init(char *dir, struct dir *org) {
   failed = anpos = 0;
-  if((orig = org) != NULL)
-    link_add(orig);
+  orig = org;
   if(curpathl == 0) {
     curpathl = strlen(dir)+1;
     curpath = malloc(curpathl);
-  } else if(curpathl < (int)strlen(dir)+1) {
+  } else if(curpathl <= (int)strlen(dir)) {
     curpathl = strlen(dir)+1;
     curpath = realloc(curpath, curpathl);
   }
