@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2009 Yoran Heling
+  Copyright (c) 2007-2010 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -211,9 +211,6 @@ delete_nxt:
 
 
 void delete_process() {
-  /* determine dir to open after successful delete */
-  struct dir *n = root->parent->sub != root ? root->parent->sub : root->next ? root->next : root->parent;
-
   /* confirm */
   seloption = 1;
   while(state == DS_CONFIRM && !noconfirm)
@@ -222,19 +219,13 @@ void delete_process() {
       return;
     }
 
-  /* temporarily re-add hard links, so we won't lose sizes in case we delete
-     a file of which another file outside this directory was marked as duplicate */
-  link_add(root);
-
   /* chdir */
   if(path_chdir(getpath(root->parent)) < 0) {
     state = DS_FAILED;
     lasterrno = errno;
     while(state == DS_FAILED)
-      if(input_handle(0)) {
-        link_del(root);
+      if(input_handle(0))
         return;
-      }
   }
 
   /* delete */
@@ -243,12 +234,10 @@ void delete_process() {
   if(delete_dir(root))
     browse_init(root);
   else {
-    if(nextsel)
-      nextsel->flags |= FF_BSEL;
-    browse_init(n);
+    nextsel->flags |= FF_BSEL;
+    browse_init(nextsel);
+    dirlist_top(-4);
   }
-  if(n != NULL)
-    link_del(n);
 }
 
 
