@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2009 Yoran Heling
+  Copyright (c) 2007-2010 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -78,11 +78,5 @@ void freedir(struct dir *);
    returned pointer will be overwritten with a subsequent call */
 char *getpath(struct dir *);
 
-/* removes all hard links from a tree */
-void link_del(struct dir *);
-
-/* re-adds all hard links in a tree */
-void link_add(struct dir *);
-
 #endif
 
